import ReactDOM from 'react-dom';
import React from 'react';

import Client from './Client';
import Login from './Login';
import Log from './Log';
import Input from './Input';

import VerticalCentering from './ui/VerticalCentering';

import style from './Application.styl';

class ApplicationView extends React.Component {
  constructor(rootContainer) {
    super();
    this.rootContainer = rootContainer;
    this.client = new Client();
    this.client.on('open', () => {
      this.setState({
        isLoggedIn: true,
        isLoggingIn: false,
        log: [],
      });
    });
    this.client.on('close', () => {
      this.setState({
        isLoggedIn: false,
        isLoggingIn: false,
        log: [],
      });
    });
    this.client.on('message', (e) => {
      this.addLogLine(e, true);
    });
    this.client.on('error', e => alert(e));
    this.state = {
      isLoggedIn: false,
      isLoggingIn: false,
      log: [],
    };
    this.handleLoginSubmit = this.handleLoginSubmit.bind(this);
    this.handleCommandSubmit = this.handleCommandSubmit.bind(this);
    this.handleDisconnectClick = this.handleDisconnectClick.bind(this);
  }

  client = null;

  addLogLine(text, isServer) {
    this.setState(old => ({
      ...old,
      log: [
        ...old.log,
        { text, isServer },
      ],
    }));
  }

  handleDisconnectClick() {
    this.client.close();
  }

  handleCommandSubmit({
    command,
  }) {
    this.addLogLine(command, false);
    this.client.send(command);
  }

  handleLoginSubmit({
    hostname,
    password,
    port = 11776,
  }) {
    this.setState({
      isLoggedIn: false,
      isLoggingIn: true,
    });
    this.client.connect(hostname, { password, port });
  }

  render() {
    let content = null;
    if (!this.state.isLoggedIn) {
      content = (
        <Login onSubmit={this.handleLoginSubmit} />
      );
    } else if (this.state.isLoggingIn) {
      content = (
        <div>Please wait...</div>
      );
    } else {
      // logged in
      content = (
        <div className="client">
          <button onClick={this.handleDisconnectClick}>Disconnect</button>
          <Input onSubmit={this.handleCommandSubmit} />
          <Log log={this.state.log} />
        </div>
      );
    }
    return (
      <div className={style.app}>
        <VerticalCentering>
          { content }
        </VerticalCentering>
      </div>
    );
  }
}

export default class Application {
  constructor(rootContainer) {
    this.rootContainer = rootContainer;
    this.elem = <ApplicationView />;
  }

  renderToContainer() {
    ReactDOM.render(this.elem, this.rootContainer);
  }
}
