import EventEmitter from 'events';

export default class Client extends EventEmitter {
  constructor() {
    super();

    this.ws = null;
  }

  isConnected() {
    return this.ws != null;
  }

  connect(hostname, {
    port = 11776,
    secure = false,
    protocols = ['dew-rcon'],
    password = '',
  }) {
    const protocol = secure ? 'wss' : 'ws';
    const url = `${protocol}://${hostname}:${port}`;
    const ws = new WebSocket(url, ...protocols);
    const hasAuthenticated = false;

    ws.onmessage = ({ data }) => {
      if (!hasAuthenticated) {
        // authentication result
        switch (data) {
          case 'deny':
            this.emit('error', 'Authentication failed');
            ws.close();
            break;
          case 'accept':
            this.ws = ws;
            this.emit('open');
            break;
          default:
            this.emit('message', data);
            break;
        }
      } else {
        this.emit('message', data);
      }
    };

    ws.onopen = () => {
      ws.send(password);
    };

    ws.onerror = (err) => {
      this.emit('error', err);
    };

    ws.onclose = (close) => {
      const { reason, ...otherErr } = close;
      this.emit('close', reason, otherErr);
    };
  }

  send(cmd) {
    this.ws.send(cmd);
  }

  close() {
    this.ws.close();
  }
}
